/* htable.h -- Brief hash ADO lookup implementation
 *
 * Author: Dylan Muir
 * Date: 9th February, 1999
 * Modified:
 * Version: 0.01
 */

#ifndef	__HTABLE_H
#define	__HTABLE_H

#include "stddefs.h"

/* -- Hash system constants -- */

#define	CASE_SENSITIVE		FALSE		/* Case sensitive hash? */
#define	MAX_KEY_SIZE		10			/* Maximum key size */
#define	HASH_EMPTY			"00"		/* Empty hash entry */
#define	HASH_DELETED		"DEL"		/* Deleted hash entry */

/* -- End hash system constants -- */
/* -- Hash system typedefs -- */

typedef	int	hEntry;

/* -- End hash system typedefs -- */
/* -- Hash system functions -- */

/* -- Function hashInitTable
 * Pre: table does not exist
 * Post: table was initialised && returen TRUE
 */
bool	hashInitTable(unsigned hashSize);


/* -- Function hashDestroyTable
 * Pre: table exists
 * Post: table was destroyed && returen TRUE
 */
bool	hashDestroyTable(void);


/* -- Function hashInsert
 * Pre: 'key' is a valid key not in table && table not full
 * Post: ('key' in table && returned TRUE) || (error && returned FALSE)
 */
bool	hashInsert(char *key, hEntry index);


/* -- Function hashValue
 * Pre: 'key' is a valid key in table
 * Post: hash 'key' --> 'value' && returned 'value'
 */
hEntry	hashValue(char *key);


/* -- Function hashDelete
 * Pre: 'key' is a valid key in table && table is not empty
 * Post: ('key' not in table && returned TRUE) || (error && returned FALSE)
 */
bool	hashDelete(char *key);


/* -- Function hashLoadFactor
 * Pre: hash system is initiallised
 * Post: the current load factor is returned as an unsigned integer
 * 		percentage from 0 to 100
 */
unsigned	hashLoadFactor(void);


/* -- Function hashNumEntries
 * Pre: hash system is initiallised
 * Post: the current number of entries is returned
 */
unsigned	hashNumEntries(void);


/* -- Function hashIsIn
 * Pre: table exists
 * Post: ('key' in table && returned TRUE) || ('key' not in table && returned FALSE)
 */
bool	hashIsIn(char *key);


/* -- Function hashIsFull
 * Pre: hash system in initiallised
 * Post: (hash table is full && returned TRUE) || returned FALSE
 */
bool hashIsFull(void);


/* -- Function hashIsEmpty
 * Pre: hash system in initiallised
 * Post: (hash table is empty && returned TRUE) || returned FALSE
 */
bool hashIsEmpty(void);


/* -- Function hashIsValidKey
 * Pre: hash system in initiallised
 * Post: ('key' is valid && returned TRUE) || returned FALSE
 */
bool hashIsValidKey(char *key);


#endif	/* __HTABLE_H */

/* --- END of htable.h --- */
